@@ -1,5 +1,5 @@
-#ifndef _LTTNG_TRACER_H
-#define _LTTNG_TRACER_H
+#ifndef _LTTNG_UST_TRACER_H
+#define _LTTNG_UST_TRACER_H
 
 /*
  * Copyright (C) 2005-2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
@@ -17,6 +17,14 @@
  *
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include <lttng/ust-config.h>
@@ -39,4 +47,4 @@
 
 #define lttng_is_signed_type(type)           (((type)(-1)) < 0)
 
-#endif /* _LTTNG_TRACER_CORE_H */
+#endif /* _LTTNG_UST_TRACER_H */
