@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009  Pierre-Marc Fournier
- * Copyright (C) 2011  Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * Copyright (C) 2011-2012  Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,8 +28,6 @@
 
 #include <lttng/ust.h>
 
-struct user_desc;
-
 pid_t fork(void)
 {
 	static pid_t (*plibc_func)(void) = NULL;
@@ -40,6 +38,7 @@ pid_t fork(void)
 		plibc_func = dlsym(RTLD_NEXT, "fork");
 		if (plibc_func == NULL) {
 			fprintf(stderr, "libustfork: unable to find \"fork\" symbol\n");
+			errno = ENOSYS;
 			return -1;
 		}
 	}
@@ -56,6 +55,38 @@ pid_t fork(void)
 	return retval;
 }
 
+int daemon(int nochdir, int noclose)
+{
+	static int (*plibc_func)(int nochdir, int noclose) = NULL;
+	sigset_t sigset;
+	int retval;
+
+	if (plibc_func == NULL) {
+		plibc_func = dlsym(RTLD_NEXT, "daemon");
+		if (plibc_func == NULL) {
+			fprintf(stderr, "libustfork: unable to find \"daemon\" symbol\n");
+			errno = ENOSYS;
+			return -1;
+		}
+	}
+
+	ust_before_fork(&sigset);
+	/* Do the real daemon call */
+	retval = plibc_func(nochdir, noclose);
+	if (retval == 0) {
+		/* child, parent called _exit() directly */
+		ust_after_fork_child(&sigset);
+	} else {
+		/* on error in the parent */
+		ust_after_fork_parent(&sigset);
+	}
+	return retval;
+}
+
+#ifdef __linux__
+
+struct user_desc;
+
 struct ustfork_clone_info {
 	int (*fn)(void *);
 	void *arg;
@@ -94,6 +125,7 @@ int clone(int (*fn)(void *), void *child_stack, int flags, void *arg, ...)
 		plibc_func = dlsym(RTLD_NEXT, "clone");
 		if (plibc_func == NULL) {
 			fprintf(stderr, "libustfork: unable to find \"clone\" symbol.\n");
+			errno = ENOSYS;
 			return -1;
 		}
 	}
@@ -107,7 +139,7 @@ int clone(int (*fn)(void *), void *child_stack, int flags, void *arg, ...)
 				tls, ctid);
 	} else {
 		/* Creating a real process, we need to intervene. */
-		struct ustfork_clone_info info = { fn = fn, arg = arg };
+		struct ustfork_clone_info info = { .fn = fn, .arg = arg };
 
 		ust_before_fork(&info.sigset);
 		retval = plibc_func(clone_fn, child_stack, flags, &info,
@@ -117,3 +149,42 @@ int clone(int (*fn)(void *), void *child_stack, int flags, void *arg, ...)
 	}
 	return retval;
 }
+
+#elif defined (__FreeBSD__)
+
+pid_t rfork(int flags)
+{
+	static pid_t (*plibc_func)(void) = NULL;
+	sigset_t sigset;
+	pid_t retval;
+
+	if (plibc_func == NULL) {
+		plibc_func = dlsym(RTLD_NEXT, "rfork");
+		if (plibc_func == NULL) {
+			fprintf(stderr, "libustfork: unable to find \"rfork\" symbol\n");
+			errno = ENOSYS;
+			return -1;
+		}
+	}
+
+	ust_before_fork(&sigset);
+	/* Do the real rfork */
+	retval = plibc_func();
+	if (retval == 0) {
+		/* child */
+		ust_after_fork_child(&sigset);
+	} else {
+		ust_after_fork_parent(&sigset);
+	}
+	return retval;
+}
+
+/*
+ * On BSD, no need to override vfork, because it runs in the context of
+ * the parent, with parent waiting until execve or exit is executed in
+ * the child.
+ */
+
+#else
+#warning "Unknown OS. You might want to ensure that fork/clone/vfork/fork handling is complete."
+#endif
