@@ -20,6 +20,14 @@ extern "C" {
  *
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include <lttng/tracepoint.h>
@@ -46,6 +54,7 @@ TRACEPOINT_EVENT(ust_tests_hello, tptest,
 		ctf_float(float, floatfield, floatarg)
 		ctf_float(double, doublefield, doublearg)
 		ctf_integer(bool, boolfield, boolarg)
+		ctf_integer_nowrite(int, filterfield, anint)
 	)
 )
 
@@ -56,12 +65,12 @@ TRACEPOINT_EVENT(ust_tests_hello, tptest_sighandler,
 
 #endif /* _TRACEPOINT_UST_TESTS_HELLO_H */
 
-#undef TRACEPOINT_INCLUDE_FILE
-#define TRACEPOINT_INCLUDE_FILE ./ust_tests_hello.h
+#undef TRACEPOINT_INCLUDE
+#define TRACEPOINT_INCLUDE "./ust_tests_hello.h"
 
 /* This part must be outside ifdef protection */
 #include <lttng/tracepoint-event.h>
 
-#ifdef __cplusplus 
+#ifdef __cplusplus
 }
 #endif
